
# How to add new Events

* Add entry to [`EventCode`](./common/v1/event_code.proto) enum.
    * Add proper namespace to your event code, e.g. `ADS_XXX`, e.g. `ADS_VIDEO_AUDIO_ROLL_REQUESTED`. Available namespace prefixes are in `conventions` section down below
    * Assign first not occupied index to new enum entry
    * Event Name should follow `Context => Object => Action` convention e.g. `ADS_VIDEO_AUDIO_ROLL_REQUESTED`. keep the 'past form' of the code, e.g. don't use `ADS_VIDEO_AUDIO_ROLL_REQUEST`, but `ADS_VIDEO_AUDIO_ROLL_REQUESTED` instead
    
    Example:
    ```
        enum EventCode {
          EVENT_CODE_UNSPECIFIED = 0;
        ...
    
        // Your events codes
          ADS_NEW_EVENT = 13;
    }
    ```

* Create protobuf file for the event:
    * Add a file under `tunein/clarity/ueapi/events/<domain>/v1/`. Eg events related to ads go to [`tunein/clarity/ueapi/events/ads/v1`](./events/ads/v1)
    * File name use lowercased event code, e.g. `ads_video_audio_roll_requested.proto`
    * Add the correct package descriptor: eg `package tunein.clarity.ueapi.events.ads.v1;` for ads domain events
    * Message name should be camelcased event code, with `Event` added at the end. . eg `ADS_DISPLAY_CLICKED` -> `AdsDisplayClickedEvent`. 
    * Add common attributes to your event and corresponding imports:
    ```
        import "google/protobuf/timestamp.proto";
        import "tunein/clarity/ueapi/common/v1/context.proto";
        import "tunein/clarity/ueapi/common/v1/event_code.proto";
        import "tunein/clarity/ueapi/common/v1/event_type.proto";
        
        message YourNewEventEvent {
              string message_id = 2; // auto generated UUID
              google.protobuf.Timestamp event_ts = 3;
              tunein.clarity.ueapi.common.v1.EventType type = 6; // EventType.EVENT_TYPE_TRACK -> change to proper type
              tunein.clarity.ueapi.common.v1.EventCode event = 7; // EventCode.ADS_XXXX -> change to your new event code
              tunein.clarity.ueapi.common.v1.Context context = 8;
            
              string attr_of_event = 20;
              bool is_another_attr_of_event = 21;
        
        }
    ```
    **Note**: Older events have `device_id` and `session_id` fields in them. They have since been moved to the Context. No need to add them to new events.
    * Add event specific attributes. Start with index 20 as in the example : [AdsPlaybackStartedEvent](./events/ads/v1/ads_playback_started.proto)
    * If new event indicates any failure on client side add common 'failure description attributes'. Example: [`AdsPlaybackFailedEvent`](./events/ads/v1/ads_playback_failed.proto)
    * If relevant, add schema documentation / comments to new event proto file
    * Try to reuse attribute fields used in other events when it makes sense. You can find list of all attributes here [Event](./v1/event.proto)
    
* Add attributes to common Event  -> [event.proto](./v1/event.proto)
    * If any of the attributes in the new event is not present in [event.proto](./v1/event.proto) then please add them there with incremented indices
    
* Add / update documentation (topology / samples etc)
    * Create doc file if not exist - file name convention `topology_xxxx.md'. Example: [topology_preroll_events.md](topology_preroll_events.md)
        * Add link to your events topology documentation in main [Readme file](../../../../README.md)
    * If doc file already exist, document new events. Example: [topology_preroll_events.md](topology_preroll_events.md)

* Submit a PR for staging repo, please mention any relevant details like JIRA links, PRD and docs.

### conventions

* Event code namespaces
    You should use one of the following namespaces if your event corresponds to the domain, or add new if needed:
    * `ADS_`
    * `USER_`
    * `LISTEN_`
    * `APP_`
    * `CONTENT_`

### Resources
1. [Protobuf Style Guide](https://protobuf.dev/programming-guides/style/)
2. [Protobuf Schema Documentation](https://protobuf.dev/programming-guides/proto3/)
